﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Globalization;
using System.IO;
using System.Threading;
using System.Windows.Forms;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.DesktopEdition;

namespace MapTest
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
            Thread.CurrentThread.CurrentCulture = CultureInfo.InvariantCulture;
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            // Set the Map Unit. The reason for setting it to DecimalDegrees is that is what the shapefile’s unit of measure is inherently in.
            winformsMap1.MapUnit = GeographyUnit.DecimalDegree;

            // Set a proper extent for the Map.  
            winformsMap1.CurrentExtent = new RectangleShape(-20, 51, 50, 28);

            // We create a new Layer and pass the path to a Shapefile into its constructor. 
            ShapeFileFeatureLayer worldLayer = new ShapeFileFeatureLayer(@"Data\WorldMap.shp");

            // Set the worldLayer with a preset Style, as AreaStyles.Country1 has YellowGreen background and black border, our worldLayer will have the same render style.  
            worldLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.Country1;

            // This setting will apply from ZoonLevel01 to ZoomLevel20, that means we can see the world the same style with ZoomLevel01 all the time no matter how far we zoom out/in. 
            worldLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            // Create a new Layer Overlay to hold the layer we just created
            LayerOverlay layerOverlay = new LayerOverlay();

            // Add the contour line layer to the layer overlay
            layerOverlay.Layers.Add(CreateContourLayer());

            // Add the shapefile layer to the layer overlay
            layerOverlay.Layers.Add(worldLayer);

            // We need to add the layer overlay to Map.
            winformsMap1.Overlays.Add(layerOverlay);

            // We now need to call the Refresh() method of the Map control so that the Map can redraw based on the data that has been provided.
            winformsMap1.Refresh();
        }

        private GridIsoLineLayer CreateContourLayer()
        {
            var data = GetContourDataFromCSV(@"Data\Winds.csv");
            CreateInterpolationGrid(data);

            GridIsoLineLayer layer = new GridIsoLineLayer(@"Data\test.grid");

            Collection<double> levels = CreateLevels(data);
            Collection<GeoColor> colors = GeoColor.GetColorsInQualityFamily(GeoColor.StandardColors.Green, GeoColor.StandardColors.Red, levels.Count + 1, ColorWheelDirection.Clockwise);

            ClassBreakStyle classBreakLineStyle = new ClassBreakStyle(layer.DataValueColumnName);
            LineStyle firstStyle = new LineStyle(new GeoPen(colors[0], 2));
            classBreakLineStyle.ClassBreaks.Add(new ClassBreak(double.MinValue, firstStyle));

            for (int i = 0; i < levels.Count - 1; ++i)
            {
                LineStyle style = new LineStyle(new GeoPen(colors[i + 1], 2));
                classBreakLineStyle.ClassBreaks.Add(new ClassBreak(levels[i], style));
            }
            layer.CustomStyles.Add(classBreakLineStyle);

            //Create the text styles to label the lines
            TextStyle textStyle = TextStyles.CreateSimpleTextStyle(layer.DataValueColumnName, "Arial", 8, DrawingFontStyles.Bold, GeoColor.StandardColors.Black, 0, 0);
            textStyle.OverlappingRule = LabelOverlappingRule.NoOverlapping;
            textStyle.SplineType = SplineType.StandardSplining;
            textStyle.DuplicateRule = LabelDuplicateRule.UnlimitedDuplicateLabels;
            textStyle.TextLineSegmentRatio = 9999999;
            textStyle.FittingLineInScreen = true;
            textStyle.SuppressPartialLabels = true;
            layer.CustomStyles.Add(textStyle);

            return layer;
        }

        private Collection<double> CreateLevels(Dictionary<PointShape, double> data)
        {
            Collection<double> levels = new Collection<double>();
            double maxLevelValue = data.Values.Max();
            double level = 0;

            do
            {
                levels.Add(level);
                level += 2;
            }
            while (level < maxLevelValue);

            return levels;
        }

        private void CreateInterpolationGrid(Dictionary<PointShape, double> contourData)
        {
            RectangleShape currentDrawingExtent = ExtentHelper.GetDrawingExtent(winformsMap1.CurrentExtent, winformsMap1.Width, winformsMap1.Height);
            double cellSize = Math.Min(currentDrawingExtent.Width / 80, currentDrawingExtent.Height / 80);

            //Greate the grid definition based on the extent, cell size etc.
            GridDefinition gridDefinition = new GridDefinition(currentDrawingExtent, cellSize, -9999, contourData);

            FileStream gridFileStream = null;

            try
            {
                gridFileStream = new FileStream(@"Data\test.grid", FileMode.Create, FileAccess.ReadWrite);
                LinealKrigingGridInterpolationModel interpolationModel = new LinealKrigingGridInterpolationModel(contourData, 5);
                GridFeatureSource.GenerateGrid(gridDefinition, interpolationModel, gridFileStream);
            }
            finally
            {
                if (gridFileStream != null) { gridFileStream.Close(); }
            }
        }

        private Dictionary<PointShape, double> GetContourDataFromCSV(string csvFilePath)
        {
            StreamReader streamReader = null;
            Dictionary<PointShape, double> contourPointShapes = new Dictionary<PointShape, double>();

            try
            {
                streamReader = new StreamReader(csvFilePath);
                while (!streamReader.EndOfStream)
                {
                    string line = streamReader.ReadLine();

                    string[] parts = line.Split(',');
                    double value = double.Parse(parts[2]);

                    PointShape point = new PointShape(double.Parse(parts[0]), double.Parse(parts[1]));
                    contourPointShapes.Add(new PointShape(point.X, point.Y), value);
                }
            }
            catch (Exception e)
            {
            }
            finally
            {
                if (streamReader != null) { streamReader.Close(); }
            }
            return contourPointShapes;
        }
    }
}
